"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getImportOrExportClause = void 0;
var typescript_1 = require("typescript");
var general_utils_1 = require("./general-utils");
var ts_type_conversion_1 = require("./ts-type-conversion");
/* ****************************************************************************************************************** */
// region: Transformer Factories
/* ****************************************************************************************************************** */
/**
 * Creates a typescript transformer which removes all statements except import / export declarations (and the module
 * nodes which contain them). Also renames the 'original' property in order to preserve type elision.
 *
 * See notes at the top of this file for more detail.
 */
function createPruneSourceFileTransformer(context) {
    var tsInstance = context.tsInstance;
    return function (ctx) {
        return function visitor(n) {
            var _a;
            if (tsInstance.isModuleDeclaration(n) || tsInstance.isSourceFile(n))
                return tsInstance.visitEachChild(n, visitor, ctx);
            if (tsInstance.isImportDeclaration(n) || tsInstance.isExportDeclaration(n)) {
                n["_baseNode"] = n.original;
                delete n.original;
                return n;
            }
            return ((_a = context.factory) !== null && _a !== void 0 ? _a : context.tsInstance).createEmptyStatement();
        };
    };
}
/**
 * Creates a typescript transformer which does no actual transformation. Rather, it uses the visitor to walk the nodes
 * and populate the nodeMap with proper linkage of original -> transformed (elided) nodes
 */
function createPopulateNodeMapTransformer(_a, nodeMap) {
    var tsInstance = _a.tsInstance;
    return function (ctx) {
        return function visitor(n) {
            var _a, _b, _c;
            if (tsInstance.isModuleDeclaration(n) || tsInstance.isSourceFile(n))
                return tsInstance.visitEachChild(n, visitor, ctx);
            if (tsInstance.isImportDeclaration(n) || tsInstance.isExportDeclaration(n)) {
                var baseNode = (_c = (_b = (_a = n.original) === null || _a === void 0 ? void 0 : _a._baseNode) !== null && _b !== void 0 ? _b : n.original) !== null && _c !== void 0 ? _c : n;
                if (baseNode)
                    nodeMap.set(baseNode, n);
            }
            return n;
        };
    };
}
function getImportOrExportClause(context, node) {
    var tsInstance = context.tsInstance, tsThreeInstance = context.tsThreeInstance, isDeclarationFile = context.isDeclarationFile, sourceFile = context.sourceFile, elisionMap = context.elisionMap, transformationContext = context.transformationContext, compilerOptions = context.compilerOptions, factory = context.factory;
    if (isDeclarationFile)
        return tsInstance.isImportDeclaration(node) ? node.importClause : node.exportClause;
    /* Prepare elision map for SourceFile if not already generated */
    if (!elisionMap.has(sourceFile)) {
        var emitResolver = transformationContext.getEmitResolver();
        var transformCompilerOptions = __assign(__assign({}, compilerOptions), { plugins: undefined });
        var nodeMap_1 = new Map();
        elisionMap.set(sourceFile, nodeMap_1);
        // Create transformers in order
        var transformers = [
            createPruneSourceFileTransformer(context),
            tsInstance.transformTypeScript,
            createPopulateNodeMapTransformer(context, nodeMap_1),
        ];
        /* Transform nodes using TS compiler API to get proper elision output */
        if (factory) {
            var transformFactory = tsInstance.createNodeFactory(typescript_1.NodeFactoryFlags.None, tsInstance.createBaseNodeFactory());
            tsInstance.transformNodes(emitResolver, 
            /* host */ void 0, transformFactory, transformCompilerOptions, 
            /* nodes */ [sourceFile], transformers, 
            /* allowDtsFiles */ false);
        }
        else {
            tsThreeInstance.transformNodes.apply(undefined, ts_type_conversion_1.downSampleTsTypes(emitResolver, 
            /* host */ void 0, transformCompilerOptions, 
            /* nodes */ [sourceFile], general_utils_1.cast(transformers), 
            /* allowDtsFiles */ false));
        }
    }
    /* Find matching node with proper elision */
    var nodeMap = elisionMap.get(sourceFile);
    var transformedMatchNode = nodeMap.get(node);
    // Match is undefined if entire declaration is elided
    if (!transformedMatchNode)
        return void 0;
    var maybeClause = tsInstance.isImportDeclaration(transformedMatchNode)
        ? transformedMatchNode.importClause
        : transformedMatchNode.exportClause;
    return maybeClause && general_utils_1.cloneNode(context, maybeClause);
}
exports.getImportOrExportClause = getImportOrExportClause;
// endregion
