"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nodeVisitor = void 0;
var utils_1 = require("./utils");
/* ****************************************************************************************************************** *
 * Node Visitor
 * ****************************************************************************************************************** */
/**
 * Visit and replace nodes with module specifiers
 */
function nodeVisitor(node) {
    var _this = this;
    var _a = this, factory = _a.factory, tsInstance = _a.tsInstance, tsThreeInstance = _a.tsThreeInstance, transformationContext = _a.transformationContext;
    /* ********************************************************* *
     * Helpers
     * ********************************************************* */
    var isAsyncImport = function (node) {
        return tsInstance.isCallExpression(node) &&
            node.expression.kind === tsInstance.SyntaxKind.ImportKeyword &&
            tsInstance.isStringLiteral(node.arguments[0]) &&
            node.arguments.length === 1;
    };
    var isRequire = function (node) {
        return tsInstance.isCallExpression(node) &&
            tsInstance.isIdentifier(node.expression) &&
            node.expression.text === "require" &&
            tsInstance.isStringLiteral(node.arguments[0]) &&
            node.arguments.length === 1;
    };
    /* ********************************************************* *
     * Visit Logic
     * ********************************************************* */
    /**
     * Update require / import functions
     *
     * require('module')
     * import('module')
     */
    if (isRequire(node) || isAsyncImport(node))
        return utils_1.resolvePathAndUpdateNode(this, node, node.arguments[0].text, function (p) {
            var tsThreeNode = utils_1.downSampleTsType(node);
            var tsThreeP = utils_1.downSampleTsType(p);
            var res = factory
                ? factory.updateCallExpression(node, node.expression, node.typeArguments, [p])
                : tsThreeInstance.updateCall(tsThreeNode, tsThreeNode.expression, tsThreeNode.typeArguments, [tsThreeP]);
            var textNode = node.arguments[0];
            var commentRanges = tsInstance.getLeadingCommentRanges(textNode.getFullText(), 0) || [];
            for (var _i = 0, commentRanges_1 = commentRanges; _i < commentRanges_1.length; _i++) {
                var range = commentRanges_1[_i];
                var kind = range.kind, pos = range.pos, end = range.end, hasTrailingNewLine = range.hasTrailingNewLine;
                var caption = textNode
                    .getFullText()
                    .substr(pos, end)
                    .replace(
                /* searchValue */ kind === tsInstance.SyntaxKind.MultiLineCommentTrivia
                    ? // Comment range in a multi-line comment with more than one line erroneously
                        // includes the node's text in the range. For that reason, we use the greedy
                        // selector in capture group and dismiss anything after the final comment close tag
                        /^\/\*(.+)\*\/.*/s
                    : /^\/\/(.+)/s, 
                /* replaceValue */ "$1");
                tsInstance.addSyntheticLeadingComment(p, kind, caption, hasTrailingNewLine);
            }
            return res;
        });
    /**
     * Update ExternalModuleReference
     *
     * import foo = require("foo");
     */
    if (tsInstance.isExternalModuleReference(node) && tsInstance.isStringLiteral(node.expression))
        return utils_1.resolvePathAndUpdateNode(this, node, node.expression.text, function (p) {
            return factory
                ? factory.updateExternalModuleReference(node, p)
                : tsThreeInstance.updateExternalModuleReference.apply(void 0, utils_1.downSampleTsTypes(node, p));
        });
    /**
     * Update ImportTypeNode
     *
     * typeof import("./bar");
     */
    if (tsInstance.isImportTypeNode(node)) {
        var argument_1 = node.argument;
        if (!tsInstance.isStringLiteral(argument_1.literal))
            return node;
        var text = argument_1.literal.text;
        if (!text)
            return node;
        return utils_1.resolvePathAndUpdateNode(this, node, text, function (p) {
            if (factory)
                return factory.updateImportTypeNode(node, factory.updateLiteralTypeNode(argument_1, p), node.qualifier, node.typeArguments, node.isTypeOf);
            var _a = utils_1.downSampleTsTypes(node, p, argument_1), tsThreeNode = _a[0], tsThreeP = _a[1], tsThreeArgument = _a[2];
            return tsThreeInstance.updateImportTypeNode(tsThreeNode, tsThreeInstance.updateLiteralTypeNode(tsThreeArgument, tsThreeP), tsThreeNode.qualifier, tsThreeNode.typeArguments, tsThreeNode.isTypeOf);
        });
    }
    /**
     * Update ImportDeclaration
     *
     * import ... 'module';
     */
    if (tsInstance.isImportDeclaration(node) && node.moduleSpecifier && tsInstance.isStringLiteral(node.moduleSpecifier))
        return utils_1.resolvePathAndUpdateNode(this, node, node.moduleSpecifier.text, function (p) {
            var importClause = node.importClause;
            if (!_this.isDeclarationFile && importClause) {
                var updatedImportClause = utils_1.getImportOrExportClause(_this, node);
                if (!updatedImportClause)
                    return undefined; // No imports left, elide entire declaration
                importClause = updatedImportClause;
            }
            if (factory)
                return factory.updateImportDeclaration(node, node.decorators, node.modifiers, importClause, p);
            var _a = utils_1.downSampleTsTypes(node, p, importClause), tsThreeNode = _a[0], tsThreeP = _a[1], tsThreeImportClause = _a[2];
            return tsThreeInstance.updateImportDeclaration(tsThreeNode, tsThreeNode.decorators, tsThreeNode.modifiers, tsThreeImportClause, tsThreeP);
        });
    /**
     * Update ExportDeclaration
     *
     * export ... 'module';
     */
    if (tsInstance.isExportDeclaration(node) && node.moduleSpecifier && tsInstance.isStringLiteral(node.moduleSpecifier))
        return utils_1.resolvePathAndUpdateNode(this, node, node.moduleSpecifier.text, function (p) {
            var exportClause = node.exportClause;
            if (!_this.isDeclarationFile && exportClause) {
                var updatedExportClause = utils_1.getImportOrExportClause(_this, node);
                if (!updatedExportClause)
                    return undefined; // No export left, elide entire declaration
                exportClause = updatedExportClause;
            }
            if (factory)
                return factory.updateExportDeclaration(node, node.decorators, node.modifiers, node.isTypeOnly, exportClause, p);
            var _a = utils_1.downSampleTsTypes(node, p, exportClause), tsThreeNode = _a[0], tsThreeP = _a[1], tsThreeExportClause = _a[2];
            return tsThreeInstance.updateExportDeclaration(tsThreeNode, tsThreeNode.decorators, tsThreeNode.modifiers, tsThreeExportClause, tsThreeP, 
            // @ts-ignore - This was added in later versions of 3.x
            tsThreeNode.isTypeOnly);
        });
    return tsInstance.visitEachChild(node, this.getVisitor(), transformationContext);
}
exports.nodeVisitor = nodeVisitor;
